@@ -15,6 +15,9 @@
 #include	<ctype.h>
 #include	<strings.h>
 #include	<time.h>
+#include	<errno.h>
+#include	<sys/stat.h>
+#include	<sys/types.h>
 #include	"builtin.h"
 
 NamespacePtr FileNamespace;
@@ -88,6 +91,14 @@ import_File_namespace()
 	    "\n"
 	    " Return whether 'f' is associated with an interactive\n"
 	    " terminal device\n" },
+	{ do_File_unlink, "unlink", "v", "s", "\n"
+	    " void unlink (string name)\n"
+	    "\n"
+	    " Delete the filename 'name'\n" },
+	{ do_File_rmdir, "rmdir", "v", "s", "\n"
+	    " void rmdir (string name)\n"
+	    "\n"
+	    " Delete the directory 'name'\n" },
         { 0 }
     };
 
@@ -127,6 +138,14 @@ import_File_namespace()
 	    " int ungetc (int c, file f)\n"
 	    "\n"
 	    " Pushes the character 'c' back on file 'f'.\n" },
+	{ do_File_rename, "rename", "v", "ss", "\n"
+	    " void rename (string oldname, string newname)\n"
+	    "\n"
+	    " Renames a file\n" },
+	{ do_File_mkdir, "mkdir", "v", "si", "\n"
+	    " void mkdir (string name, int mode)\n"
+	    "\n"
+	    " Create the new directory 'name'\n" },
         { 0 }
     };
 
@@ -181,6 +200,18 @@ import_File_namespace()
 	    " io_error (string message, error_type error, file f)\n"
 	    "\n"
 	    " Raised when an i/o error occurs.\n" },
+	{"name_error",		exception_name_error,		"sEs", "\n"
+	    " name_error (string message, error_type error, string name)\n"
+	    "\n"
+	    " Raised when an operation involving a filename fails.\n"
+	    " 'message' is a printable error string.\n"
+	    " 'error' is a symbolic error code.\n"
+	    " 'name' is the filename which failed.\n" },
+	{"io_eof",		exception_io_eof,		"f", "\n"
+	    " io_eof (file f)\n"
+	    "\n"
+	    " Raised when reading at end-of-file.\n"
+	    " 'file' is the file at eof.\n" },
 	{0,			0 },
     };
 
@@ -218,8 +249,9 @@ do_File_print (Value file, Value value, Value format,
 	return Void;
     if (ibase < 0 || ibase == 1)
     {
-	RaiseStandardException (exception_invalid_argument,
-				"Illegal base", 2, NewInt (0), base);
+	RaiseStandardException (exception_invalid_argument, 3,
+				NewStrString ("Illegal base"),
+				NewInt (0), base);
 	return Void;
     }
     iwidth = IntPart (width, "Illegal width");
@@ -241,9 +273,10 @@ do_File_print (Value file, Value value, Value format,
 			  fill->string.length, 0));
 	if (file->file.flags & FileOutputError)
 	{
-	    RaiseStandardException (exception_io_error, 
+	    RaiseStandardException (exception_io_error, 3,
 				    FileGetErrorMessage (file->file.output_errno), 
-				    2, FileGetError (file->file.output_errno), file);
+				    FileGetError (file->file.output_errno),
+				    file);
 	}
     }
     return Void;
@@ -268,9 +301,10 @@ do_File_open (Value name, Value mode)
     ret = FileFopen (n, m, &err);
     if (!ret)
     {
-	RaiseStandardException (exception_open_error,
+	RaiseStandardException (exception_open_error, 3,
 				FileGetErrorMessage (err),
-				2, FileGetError (err), name);
+				FileGetError (err),
+				name);
 	RETURN (Void);
     }
     complete = True;
@@ -285,9 +319,9 @@ do_File_flush (Value f)
 	ThreadSleep (running, f, PriorityIo);
 	break;
     case FileError:
-	RaiseStandardException (exception_io_error, 
+	RaiseStandardException (exception_io_error, 3,
 				FileGetErrorMessage (f->file.output_errno), 
-				2, FileGetError (f->file.output_errno), f);
+				FileGetError (f->file.output_errno), f);
 	break;
     }
     return Void;
@@ -303,16 +337,16 @@ do_File_close (Value f)
 	ThreadSleep (running, f, PriorityIo);
 	break;
     case FileError:
-	RaiseStandardException (exception_io_error, 
+	RaiseStandardException (exception_io_error, 3,
 				FileGetErrorMessage (f->file.output_errno), 
-				2, FileGetError (f->file.output_errno), f);
+				FileGetError (f->file.output_errno), f);
 	break;
     default:
 	if (FileClose (f) == FileError)
 	{
-	    RaiseStandardException (exception_io_error, 
+	    RaiseStandardException (exception_io_error, 3,
 				    FileGetErrorMessage (f->file.output_errno), 
-				    2, FileGetError (f->file.output_errno), f);
+				    FileGetError (f->file.output_errno), f);
 	}
 	else
 	    complete = True;
@@ -351,9 +385,9 @@ do_File_filter (Value path, Value argv, Value filev)
     ret = FileFilter (p, args, filev, &err);
     if (!ret)
     {
-	RaiseStandardException (exception_open_error,
+	RaiseStandardException (exception_open_error, 3,
 				FileGetErrorMessage (err),
-				2, FileGetError (err), path);
+				FileGetError (err), path);
 	ret = Void;
     }
     complete = True;
@@ -370,9 +404,9 @@ Value do_File_mkpipe (void) {
     ret = FileMakePipe (&err);
     if (!ret)
     {
-	RaiseStandardException (exception_open_error,
+	RaiseStandardException (exception_open_error, 3,
 				FileGetErrorMessage (err),
-				2, FileGetError (err), Void);
+				FileGetError (err), Void);
 	RETURN (Void);
     }
     RETURN (ret);
@@ -397,9 +431,9 @@ do_File_reopen (Value name, Value mode, Value file)
     ret = FileReopen (n, m, file, &err);
     if (!ret)
     {
-	RaiseStandardException (exception_open_error,
+	RaiseStandardException (exception_open_error, 3,
 				FileGetErrorMessage (err),
-				2, FileGetError (err), name);
+				FileGetError (err), name);
 	RETURN (Void);
     }
     complete = True;
@@ -451,9 +485,9 @@ do_File_getb (Value f)
 	    ThreadSleep (running, f, PriorityIo);
 	    RETURN (Void);
 	case FileError:
-	    RaiseStandardException (exception_io_error,
+	    RaiseStandardException (exception_io_error, 3,
 				    FileGetErrorMessage (f->file.input_errno),
-				    2, FileGetError (f->file.input_errno), f);
+				    FileGetError (f->file.input_errno), f);
 	    RETURN (Void);
 	default:
 	    complete = True;
@@ -477,9 +511,12 @@ do_File_getc (Value f)
 	    ThreadSleep (running, f, PriorityIo);
 	    RETURN (Void);
 	case FileError:
-	    RaiseStandardException (exception_io_error,
+	    RaiseStandardException (exception_io_error, 3,
 				    FileGetErrorMessage (f->file.input_errno),
-				    2, FileGetError (f->file.input_errno), f);
+				    FileGetError (f->file.input_errno), f);
+	    RETURN (Void);
+	case FileEOF:
+	    RaiseStandardException (exception_io_eof, 1, f);
 	    RETURN (Void);
 	default:
 	    complete = True;
@@ -541,9 +578,10 @@ do_File_putb (Value v, Value f)
 	{
 	    if (FileOutput (f, IntPart (v, "putb non integer")) == FileError)
 	    {
-		RaiseStandardException (exception_io_error,
+		RaiseStandardException (exception_io_error, 3,
 					FileGetErrorMessage (f->file.output_errno),
-					2, FileGetError (f->file.output_errno), f);
+					FileGetError (f->file.output_errno),
+					f);
 	    }
 	    else
 		complete = True;
@@ -565,9 +603,10 @@ do_File_putc (Value v, Value f)
 	{
 	    if (FileOutchar (f, IntPart (v, "putc non integer")) == FileError)
 	    {
-		RaiseStandardException (exception_io_error,
+		RaiseStandardException (exception_io_error, 3,
 					FileGetErrorMessage (f->file.output_errno),
-					2, FileGetError (f->file.output_errno), f);
+					FileGetError (f->file.output_errno),
+					f);
 	    }
 	    else
 		complete = True;
@@ -618,3 +657,94 @@ do_File_setbuf (Value f, Value v)
 	FileSetBuffer (f, i);
     RETURN (v);
 }
+
+Value
+do_File_unlink (Value name)
+{
+    ENTER ();
+    char *n;
+    int ret;
+
+    n = StrzPart (name, "invalid file name");
+    if (!n)
+	RETURN (Void);
+    ret = unlink (n);
+    if (ret < 0) {
+	int err = errno;
+	RaiseStandardException (exception_name_error, 3,
+				FileGetErrorMessage (err),
+				FileGetError (err), name);
+	RETURN (Void);
+    }
+    RETURN (Void);
+}
+
+Value
+do_File_rename (Value old, Value new)
+{
+    ENTER ();
+    char *o, *n;
+    int ret;
+
+    o = StrzPart (old, "invalid file name");
+    if (!o)
+	RETURN (Void);
+    n = StrzPart (new, "invalid file name");
+    if (!n)
+	RETURN (Void);
+    ret = rename (o, n);
+    if (ret < 0) {
+	int err = errno;
+	RaiseStandardException (exception_name_error, 3,
+				FileGetErrorMessage (err),
+				FileGetError (err), new);
+	RETURN (Void);
+    }
+    RETURN (Void);
+}
+
+Value
+do_File_mkdir (Value name, Value mode)
+{
+    ENTER ();
+    char *n;
+    int m;
+    int ret;
+
+    n = StrzPart (name, "invalid file name");
+    if (!n)
+	RETURN (Void);
+    m = IntPart (mode, "invalid file mode");
+    if (aborting)
+	RETURN (Void);
+    ret = mkdir (n, m);
+    if (ret < 0) {
+	int err = errno;
+	RaiseStandardException (exception_name_error, 3,
+				FileGetErrorMessage (err),
+				FileGetError (err), name);
+	RETURN (Void);
+    }
+    RETURN (Void);
+}
+
+Value
+do_File_rmdir (Value name)
+{
+    ENTER ();
+    char *n;
+    int ret;
+
+    n = StrzPart (name, "invalid file name");
+    if (!n)
+	RETURN (Void);
+    ret = rmdir (n);
+    if (ret < 0) {
+	int err = errno;
+	RaiseStandardException (exception_name_error, 3,
+				FileGetErrorMessage (err),
+				FileGetError (err), name);
+	RETURN (Void);
+    }
+    RETURN (Void);
+}
